@@ -23,11 +23,20 @@
 #include <time.h>
 #include <ctype.h>
 
+//----------------------------------------------------------------------------
+static unsigned int cmCTestGITVersion(unsigned int epic, unsigned int major,
+                                      unsigned int minor, unsigned int fix)
+{
+  // 1.6.5.0 maps to 10605000
+  return fix + minor*1000 + major*100000 + epic*10000000;
+}
+
 //----------------------------------------------------------------------------
 cmCTestGIT::cmCTestGIT(cmCTest* ct, std::ostream& log):
   cmCTestGlobalVC(ct, log)
 {
   this->PriorRev = this->Unknown;
+  this->CurrentGitVersion = 0;
 }
 
 //----------------------------------------------------------------------------
@@ -263,13 +272,47 @@ bool cmCTestGIT::UpdateImpl()
 
   std::string top_dir = this->FindTopDir();
   const char* git = this->CommandLineTool.c_str();
-  char const* git_submodule[] = {git, "submodule", "update", 0};
+  const char* recursive = "--recursive";
+
+  // Git < 1.6.5.0 did not support --recursive
+  if(this->GetGitVersion() < cmCTestGITVersion(1,6,5,0))
+    {
+    recursive = 0;
+    // No need to require >= 1.6.5.0 if there are no submodules.
+    if(cmSystemTools::FileExists((top_dir + "/.gitmodules").c_str()))
+      {
+      this->Log << "Git < 1.6.5.0 cannot update submodules recursively\n";
+      }
+    }
+
+  char const* git_submodule[] = {git, "submodule", "update", recursive, 0};
   OutputLogger submodule_out(this->Log, "submodule-out> ");
   OutputLogger submodule_err(this->Log, "submodule-err> ");
   return this->RunChild(git_submodule, &submodule_out, &submodule_err,
                         top_dir.c_str());
 }
 
+//----------------------------------------------------------------------------
+unsigned int cmCTestGIT::GetGitVersion()
+{
+  if(!this->CurrentGitVersion)
+    {
+    const char* git = this->CommandLineTool.c_str();
+    char const* git_version[] = {git, "--version", 0};
+    std::string version;
+    OneLineParser version_out(this, "version-out> ", version);
+    OutputLogger version_err(this->Log, "version-err> ");
+    unsigned int v[4] = {0,0,0,0};
+    if(this->RunChild(git_version, &version_out, &version_err) &&
+       sscanf(version.c_str(), "git version %u.%u.%u.%u",
+              &v[0], &v[1], &v[2], &v[3]) >= 3)
+      {
+      this->CurrentGitVersion = cmCTestGITVersion(v[0], v[1], v[2], v[3]);
+      }
+    }
+  return this->CurrentGitVersion;
+}
+
 //----------------------------------------------------------------------------
 /* Diff format:
 
@@ -503,28 +546,15 @@ private:
       this->ParsePerson(this->Line.c_str()+7, author);
       this->Rev.Author = author.Name;
       this->Rev.EMail = author.EMail;
-
-      // Convert the time to a human-readable format that is also easy
-      // to machine-parse: "CCYY-MM-DD hh:mm:ss".
-      time_t seconds = static_cast<time_t>(author.Time);
-      struct tm* t = gmtime(&seconds);
-      char dt[1024];
-      sprintf(dt, "%04d-%02d-%02d %02d:%02d:%02d",
-              t->tm_year+1900, t->tm_mon+1, t->tm_mday,
-              t->tm_hour, t->tm_min, t->tm_sec);
-      this->Rev.Date = dt;
-
-      // Add the time-zone field "+zone" or "-zone".
-      char tz[32];
-      if(author.TimeZone >= 0)
-        {
-        sprintf(tz, " +%04ld", author.TimeZone);
-        }
-      else
-        {
-        sprintf(tz, " -%04ld", -author.TimeZone);
-        }
-      this->Rev.Date += tz;
+      this->Rev.Date = this->FormatDateTime(author);
+      }
+    else if(strncmp(this->Line.c_str(), "committer ", 10) == 0)
+      {
+      Person committer;
+      this->ParsePerson(this->Line.c_str()+10, committer);
+      this->Rev.Committer = committer.Name;
+      this->Rev.CommitterEMail = committer.EMail;
+      this->Rev.CommitDate = this->FormatDateTime(committer);
       }
     }
 
@@ -537,6 +567,32 @@ private:
       }
     this->Rev.Log += "\n";
     }
+
+  std::string FormatDateTime(Person const& person)
+    {
+    // Convert the time to a human-readable format that is also easy
+    // to machine-parse: "CCYY-MM-DD hh:mm:ss".
+    time_t seconds = static_cast<time_t>(person.Time);
+    struct tm* t = gmtime(&seconds);
+    char dt[1024];
+    sprintf(dt, "%04d-%02d-%02d %02d:%02d:%02d",
+            t->tm_year+1900, t->tm_mon+1, t->tm_mday,
+            t->tm_hour, t->tm_min, t->tm_sec);
+    std::string out = dt;
+
+    // Add the time-zone field "+zone" or "-zone".
+    char tz[32];
+    if(person.TimeZone >= 0)
+      {
+      sprintf(tz, " +%04ld", person.TimeZone);
+      }
+    else
+      {
+      sprintf(tz, " -%04ld", -person.TimeZone);
+      }
+    out += tz;
+    return out;
+    }
 };
 
 char const cmCTestGIT::CommitParser::SectionSep[SectionCount] =
