@@ -47,12 +47,12 @@ public:
   /**
    * The name of the command as specified in CMakeList.txt.
    */
-  virtual const char* GetName() { return "ctest_run_script";}
+  virtual const char* GetName() const { return "ctest_run_script";}
 
   /**
    * Succinct documentation.
    */
-  virtual const char* GetTerseDocumentation() 
+  virtual const char* GetTerseDocumentation() const
     {
     return "runs a ctest -S script";
     }
@@ -60,7 +60,7 @@ public:
   /**
    * More documentation.
    */
-  virtual const char* GetFullDocumentation()
+  virtual const char* GetFullDocumentation() const
     {
     return
       "  ctest_run_script([NEW_PROCESS] script_file_name script_file_name1 \n"
