@@ -1,19 +1,14 @@
-/*=========================================================================
+/*============================================================================
+  CMake - Cross Platform Makefile Generator
+  Copyright 2000-2009 Kitware, Inc., Insight Software Consortium
 
-  Program:   CMake - Cross-Platform Makefile Generator
-  Module:    $RCSfile: cmCTestUpdateCommand.cxx,v $
-  Language:  C++
-  Date:      $Date: 2006-07-11 19:58:07 $
-  Version:   $Revision: 1.15 $
+  Distributed under the OSI-approved BSD License (the "License");
+  see accompanying file Copyright.txt for details.
 
-  Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
-  See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notices for more information.
-
-=========================================================================*/
+  This software is distributed WITHOUT ANY WARRANTY; without even the
+  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+  See the License for more information.
+============================================================================*/
 #include "cmCTestUpdateCommand.h"
 
 #include "cmCTest.h"
@@ -48,14 +43,20 @@ cmCTestGenericHandler* cmCTestUpdateCommand::InitializeHandler()
     "SVNCommand", "CTEST_SVN_COMMAND");
   this->CTest->SetCTestConfigurationFromCMakeVariable(this->Makefile,
     "SVNUpdateOptions", "CTEST_SVN_UPDATE_OPTIONS");
-
-  const char* initialCheckoutCommand
-    = this->Makefile->GetDefinition("CTEST_CHECKOUT_COMMAND");
-  if ( !initialCheckoutCommand )
-    {
-    initialCheckoutCommand = 
-      this->Makefile->GetDefinition("CTEST_CVS_CHECKOUT");
-    }
+  this->CTest->SetCTestConfigurationFromCMakeVariable(this->Makefile,
+    "BZRCommand", "CTEST_BZR_COMMAND");
+  this->CTest->SetCTestConfigurationFromCMakeVariable(this->Makefile,
+    "BZRUpdateOptions", "CTEST_BZR_UPDATE_OPTIONS");
+  this->CTest->SetCTestConfigurationFromCMakeVariable(this->Makefile,
+    "GITCommand", "CTEST_GIT_COMMAND");
+  this->CTest->SetCTestConfigurationFromCMakeVariable(this->Makefile,
+    "GITUpdateOptions", "CTEST_GIT_UPDATE_OPTIONS");
+  this->CTest->SetCTestConfigurationFromCMakeVariable(this->Makefile,
+    "GITUpdateCustom", "CTEST_GIT_UPDATE_CUSTOM");
+  this->CTest->SetCTestConfigurationFromCMakeVariable(this->Makefile,
+    "HGCommand", "CTEST_HG_COMMAND");
+  this->CTest->SetCTestConfigurationFromCMakeVariable(this->Makefile,
+    "HGUpdateOptions", "CTEST_HG_UPDATE_OPTIONS");
 
   cmCTestGenericHandler* handler
     = this->CTest->GetInitializedHandler("update");
@@ -71,24 +72,6 @@ cmCTestGenericHandler* cmCTestUpdateCommand::InitializeHandler()
     return 0;
     }
   handler->SetOption("SourceDirectory", source_dir.c_str());
-  if ( initialCheckoutCommand )
-    {
-    handler->SetOption("InitialCheckout", initialCheckoutCommand);
-    }
-  if ( (!cmSystemTools::FileExists(source_dir.c_str()) ||
-      !cmSystemTools::FileIsDirectory(source_dir.c_str()))
-    && !initialCheckoutCommand )
-    {
-    cmOStringStream str;
-    str << "cannot find source directory: " << source_dir.c_str() << ".";
-    if ( !cmSystemTools::FileExists(source_dir.c_str()) )
-      {
-      str << " Looks like it is not checked out yet. Please specify "
-        "CTEST_CHECKOUT_COMMAND.";
-      }
-    this->SetError(str.str().c_str());
-    return 0;
-    }
   return handler;
 }
 
