@@ -1,19 +1,14 @@
-/*=========================================================================
+/*============================================================================
+  CMake - Cross Platform Makefile Generator
+  Copyright 2000-2009 Kitware, Inc., Insight Software Consortium
 
-  Program:   CMake - Cross-Platform Makefile Generator
-  Module:    $RCSfile: cmEnableTestingCommand.h,v $
-  Language:  C++
-  Date:      $Date: 2005/11/23 17:33:38 $
-  Version:   $Revision: 1.11.2.1 $
+  Distributed under the OSI-approved BSD License (the "License");
+  see accompanying file Copyright.txt for details.
 
-  Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
-  See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even 
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
-     PURPOSE.  See the above copyright notices for more information.
-
-=========================================================================*/
+  This software is distributed WITHOUT ANY WARRANTY; without even the
+  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+  See the License for more information.
+============================================================================*/
 #ifndef cmEnableTestingCommand_h
 #define cmEnableTestingCommand_h
 
@@ -29,7 +24,7 @@
  * within the valid control structures are replicated in Testfile
  * (i.e. SUBDIRS() and ADD_TEST() commands within IF() commands that are
  * not entered by CMake are not replicated in Testfile).
- * Note that Dart expects to find this file in the build directory root; 
+ * Note that CTest expects to find this file in the build directory root; 
  * therefore, this command should be in the source directory root too.
  */
 class cmEnableTestingCommand : public cmCommand
@@ -47,17 +42,18 @@ public:
    * This is called when the command is first encountered in
    * the CMakeLists.txt file.
    */
-  virtual bool InitialPass(std::vector<std::string> const&);
+  virtual bool InitialPass(std::vector<std::string> const&,
+                           cmExecutionStatus &);
 
   /**
    * The name of the command as specified in CMakeList.txt.
    */
-  virtual const char* GetName() { return "ENABLE_TESTING";}
+  virtual const char* GetName() const { return "enable_testing";}
 
   /**
    * Succinct documentation.
    */
-  virtual const char* GetTerseDocumentation() 
+  virtual const char* GetTerseDocumentation() const
     {
     return "Enable testing for current directory and below.";
     }
@@ -65,20 +61,17 @@ public:
   /**
    * More documentation.
    */
-  virtual const char* GetFullDocumentation()
+  virtual const char* GetFullDocumentation() const
     {
     return
-      "  ENABLE_TESTING()\n"
+      "  enable_testing()\n"
       "Enables testing for this directory and below.  "
-      "See also the ADD_TEST command.  Note that ctest expects to find "
+      "See also the add_test command.  Note that ctest expects to find "
       "a test file in the build directory root.  Therefore, this command "
       "should be in the source directory root.";
     }
   
   cmTypeMacro(cmEnableTestingCommand, cmCommand);
-
-  ///! method to recurse and write the DartTestfiles
-  void CreateDartTestfileForMakefile(cmMakefile *mf);
   
 };
 
