@@ -108,11 +108,11 @@ void cmFindBase::GenerateDocumentation()
 }
 
 //----------------------------------------------------------------------------
-const char* cmFindBase::GetFullDocumentation()
+const char* cmFindBase::GetFullDocumentation() const
 {
   if(this->GenericDocumentation.empty())
     {
-    this->GenerateDocumentation();
+    const_cast<cmFindBase *>(this)->GenerateDocumentation();
     }
   return this->GenericDocumentation.c_str();
 }
@@ -299,11 +299,7 @@ bool cmFindBase::ParseArguments(std::vector<std::string> const& argsIn)
   this->GetIgnoredPaths(ignored);
   this->FilterPaths(this->SearchPaths, ignored);
 
-  // Handle search root stuff.
-  this->RerootPaths(this->SearchPaths);
-
-  // Add a trailing slash to all prefixes to aid the search process.
-  this->AddTrailingSlashes(this->SearchPaths);
+  this->ComputeFinalPaths();
 
   return true;
 }
