@@ -57,7 +57,13 @@ void cmFindLibraryCommand::GenerateDocumentation()
     "the full path to the framework <fullPath>/A.framework. "
     "When a full path to a framework is used as a library, "
     "CMake will use a -framework A, and a -F<fullPath> to "
-    "link the framework to the target. ";
+    "link the framework to the target."
+    "\n"
+    "If the global property FIND_LIBRARY_USE_LIB64_PATHS is set all search "
+    "paths will be tested as normal, with \"64/\" appended, and with all "
+    "matches of \"lib/\" replaced with \"lib64/\". This property is "
+    "automatically set for the platforms that are known to need it if at "
+    "least one of the languages supported by the PROJECT command is enabled.";
 }
 
 // cmFindLibraryCommand
@@ -354,13 +360,23 @@ void cmFindLibraryHelper::RegexFromList(std::string& out,
 //----------------------------------------------------------------------------
 bool cmFindLibraryHelper::HasValidSuffix(std::string const& name)
 {
-  // Check if the given name ends in a valid library suffix.
   for(std::vector<std::string>::const_iterator si = this->Suffixes.begin();
       si != this->Suffixes.end(); ++si)
     {
-    std::string const& suffix = *si;
-    if(name.length() > suffix.length() &&
-       name.substr(name.size()-suffix.length()) == suffix)
+    std::string suffix = *si;
+    if(name.length() <= suffix.length())
+      {
+      continue;
+      }
+    // Check if the given name ends in a valid library suffix.
+    if(name.substr(name.size()-suffix.length()) == suffix)
+      {
+      return true;
+      }
+    // Check if a valid library suffix is somewhere in the name,
+    // this may happen e.g. for versioned shared libraries: libfoo.so.2
+    suffix += ".";
+    if(name.find(suffix) != name.npos)
       {
       return true;
       }
