@@ -1,19 +1,14 @@
-/*=========================================================================
+/*============================================================================
+  CMake - Cross Platform Makefile Generator
+  Copyright 2000-2009 Kitware, Inc., Insight Software Consortium
 
-  Program:   CMake - Cross-Platform Makefile Generator
-  Module:    $RCSfile: cmIfCommand.cxx,v $
-  Language:  C++
-  Date:      $Date: 2009-02-04 16:44:17 $
-  Version:   $Revision: 1.84.2.5 $
+  Distributed under the OSI-approved BSD License (the "License");
+  see accompanying file Copyright.txt for details.
 
-  Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
-  See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even 
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
-     PURPOSE.  See the above copyright notices for more information.
-
-=========================================================================*/
+  This software is distributed WITHOUT ANY WARRANTY; without even the
+  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+  See the License for more information.
+============================================================================*/
 #include "cmIfCommand.h"
 #include "cmStringCommand.h"
 
@@ -21,9 +16,25 @@
 #include <list>
 #include <cmsys/RegularExpression.hxx>
 
+
+static std::string cmIfCommandError(
+  cmMakefile* mf, std::vector<std::string> const& args)
+{
+  cmLocalGenerator* lg = mf->GetLocalGenerator();
+  std::string err = "given arguments:\n ";
+  for(std::vector<std::string>::const_iterator i = args.begin();
+      i != args.end(); ++i)
+    {
+    err += " ";
+    err += lg->EscapeForCMake(i->c_str());
+    }
+  err += "\n";
+  return err;
+}
+
 //=========================================================================
 bool cmIfFunctionBlocker::
-IsFunctionBlocked(const cmListFileFunction& lff, 
+IsFunctionBlocked(const cmListFileFunction& lff,
                   cmMakefile &mf,
                   cmExecutionStatus &inStatus)
 {
@@ -36,7 +47,7 @@ IsFunctionBlocked(const cmListFileFunction& lff,
     {
     this->ScopeDepth--;
     // if this is the endif for this if statement, then start executing
-    if (!this->ScopeDepth) 
+    if (!this->ScopeDepth)
       {
       // Remove the function blocker for this scope or bail.
       cmsys::auto_ptr<cmFunctionBlocker>
@@ -63,6 +74,13 @@ IsFunctionBlocked(const cmListFileFunction& lff,
           {
           this->IsBlocking = this->HasRun;
           this->HasRun = true;
+
+          // if trace is enabled, print a (trivially) evaluated "else"
+          // statement
+          if(!this->IsBlocking && mf.GetCMakeInstance()->GetTrace())
+            {
+            mf.PrintCommandTrace(this->Functions[c]);
+            }
           }
         else if (scopeDepth == 0 && !cmSystemTools::Strucmp
                  (this->Functions[c].Name.c_str(),"elseif"))
@@ -77,33 +95,35 @@ IsFunctionBlocked(const cmListFileFunction& lff,
             cmMakefileCall stack_manager(&mf, this->Functions[c], status);
             static_cast<void>(stack_manager);
 
+            // if trace is enabled, print the evaluated "elseif" statement
+            if(mf.GetCMakeInstance()->GetTrace())
+              {
+              mf.PrintCommandTrace(this->Functions[c]);
+              }
+
             std::string errorString;
-            
+
             std::vector<std::string> expandedArguments;
-            mf.ExpandArguments(this->Functions[c].Arguments, 
+            mf.ExpandArguments(this->Functions[c].Arguments,
                                expandedArguments);
-            bool isTrue = 
-              cmIfCommand::IsTrue(expandedArguments,errorString,&mf);
-            
+
+            cmake::MessageType messType;
+            bool isTrue =
+              cmIfCommand::IsTrue(expandedArguments, errorString,
+                                  &mf, messType);
+
             if (errorString.size())
               {
-              std::string err = "had incorrect arguments: ";
-              unsigned int i;
-              for(i =0; i < this->Functions[c].Arguments.size(); ++i)
-                {
-                err += (this->Functions[c].Arguments[i].Quoted?"\"":"");
-                err += this->Functions[c].Arguments[i].Value;
-                err += (this->Functions[c].Arguments[i].Quoted?"\"":"");
-                err += " ";
-                }
-              err += "(";
+              std::string err = cmIfCommandError(&mf, expandedArguments);
               err += errorString;
-              err += ").";
-              mf.IssueMessage(cmake::FATAL_ERROR, err);
-              cmSystemTools::SetFatalErrorOccured();
-              return true;
+              mf.IssueMessage(messType, err);
+              if (messType == cmake::FATAL_ERROR)
+                {
+                cmSystemTools::SetFatalErrorOccured();
+                return true;
+                }
               }
-        
+
             if (isTrue)
               {
               this->IsBlocking = false;
@@ -111,7 +131,7 @@ IsFunctionBlocked(const cmListFileFunction& lff,
               }
             }
           }
-            
+
         // should we execute?
         else if (!this->IsBlocking)
           {
@@ -132,10 +152,10 @@ IsFunctionBlocked(const cmListFileFunction& lff,
       return true;
       }
     }
-  
+
   // record the command
   this->Functions.push_back(lff);
-  
+
   // always return true
   return true;
 }
@@ -160,35 +180,35 @@ bool cmIfFunctionBlocker::ShouldRemove(const cmListFileFunction& lff,
 
 //=========================================================================
 bool cmIfCommand
-::InvokeInitialPass(const std::vector<cmListFileArgument>& args, 
+::InvokeInitialPass(const std::vector<cmListFileArgument>& args,
                     cmExecutionStatus &)
 {
   std::string errorString;
-  
+
   std::vector<std::string> expandedArguments;
   this->Makefile->ExpandArguments(args, expandedArguments);
-  bool isTrue = 
-    cmIfCommand::IsTrue(expandedArguments,errorString,this->Makefile);
-  
+
+  cmake::MessageType status;
+  bool isTrue =
+    cmIfCommand::IsTrue(expandedArguments,errorString,
+                        this->Makefile, status);
+
   if (errorString.size())
     {
-    std::string err = "had incorrect arguments: ";
-    unsigned int i;
-    for(i =0; i < args.size(); ++i)
-      {
-      err += (args[i].Quoted?"\"":"");
-      err += args[i].Value;
-      err += (args[i].Quoted?"\"":"");
-      err += " ";
-      }
-    err += "(";
+    std::string err = cmIfCommandError(this->Makefile, expandedArguments);
     err += errorString;
-    err += ").";
-    this->SetError(err.c_str());
-    cmSystemTools::SetFatalErrorOccured();
-    return false;
+    if (status == cmake::FATAL_ERROR)
+      {
+      this->SetError(err.c_str());
+      cmSystemTools::SetFatalErrorOccured();
+      return false;
+      }
+    else
+      {
+      this->Makefile->IssueMessage(status, err);
+      }
     }
-  
+
   cmIfFunctionBlocker *f = new cmIfFunctionBlocker();
   // if is isn't true block the commands
   f->ScopeDepth = 1;
@@ -199,13 +219,132 @@ bool cmIfCommand
     }
   f->Args = args;
   this->Makefile->AddFunctionBlocker(f);
-  
+
   return true;
 }
 
-namespace 
+namespace
 {
-//=========================================================================
+  //=========================================================================
+  bool GetBooleanValue(std::string& arg, cmMakefile* mf)
+  {
+  // Check basic constants.
+  if (arg == "0")
+    {
+    return false;
+    }
+  if (arg == "1")
+    {
+    return true;
+    }
+
+  // Check named constants.
+  if (cmSystemTools::IsOn(arg.c_str()))
+    {
+    return true;
+    }
+  if (cmSystemTools::IsOff(arg.c_str()))
+    {
+    return false;
+    }
+
+  // Check for numbers.
+  if(!arg.empty())
+    {
+    char* end;
+    double d = strtod(arg.c_str(), &end);
+    if(*end == '\0')
+      {
+      // The whole string is a number.  Use C conversion to bool.
+      return d? true:false;
+      }
+    }
+
+  // Check definition.
+  const char* def = mf->GetDefinition(arg.c_str());
+  return !cmSystemTools::IsOff(def);
+  }
+
+  //=========================================================================
+  // Boolean value behavior from CMake 2.6.4 and below.
+  bool GetBooleanValueOld(std::string const& arg, cmMakefile* mf, bool one)
+  {
+  if(one)
+    {
+    // Old IsTrue behavior for single argument.
+    if(arg == "0")
+      { return false; }
+    else if(arg == "1")
+      { return true; }
+    else
+      { return !cmSystemTools::IsOff(mf->GetDefinition(arg.c_str())); }
+    }
+  else
+    {
+    // Old GetVariableOrNumber behavior.
+    const char* def = mf->GetDefinition(arg.c_str());
+    if(!def && atoi(arg.c_str()))
+      {
+      def = arg.c_str();
+      }
+    return !cmSystemTools::IsOff(def);
+    }
+  }
+
+  //=========================================================================
+  // returns the resulting boolean value
+  bool GetBooleanValueWithAutoDereference(
+    std::string &newArg,
+    cmMakefile *makefile,
+    std::string &errorString,
+    cmPolicies::PolicyStatus Policy12Status,
+    cmake::MessageType &status,
+    bool oneArg = false)
+  {
+  // Use the policy if it is set.
+  if (Policy12Status == cmPolicies::NEW)
+    {
+    return GetBooleanValue(newArg, makefile);
+    }
+  else if (Policy12Status == cmPolicies::OLD)
+    {
+    return GetBooleanValueOld(newArg, makefile, oneArg);
+    }
+
+  // Check policy only if old and new results differ.
+  bool newResult = GetBooleanValue(newArg, makefile);
+  bool oldResult = GetBooleanValueOld(newArg, makefile, oneArg);
+  if(newResult != oldResult)
+    {
+    switch(Policy12Status)
+      {
+      case cmPolicies::WARN:
+        {
+        cmPolicies* policies = makefile->GetPolicies();
+        errorString = "An argument named \"" + newArg
+          + "\" appears in a conditional statement.  "
+          + policies->GetPolicyWarning(cmPolicies::CMP0012);
+        status = cmake::AUTHOR_WARNING;
+        }
+      case cmPolicies::OLD:
+        return oldResult;
+      case cmPolicies::REQUIRED_IF_USED:
+      case cmPolicies::REQUIRED_ALWAYS:
+        {
+        cmPolicies* policies = makefile->GetPolicies();
+        errorString = "An argument named \"" + newArg
+          + "\" appears in a conditional statement.  "
+          + policies->GetRequiredPolicyError(cmPolicies::CMP0012);
+        status = cmake::FATAL_ERROR;
+        }
+      case cmPolicies::NEW:
+        break;
+      }
+    }
+  return newResult;
+  }
+
+  //=========================================================================
   void IncrementArguments(std::list<std::string> &newArgs,
                           std::list<std::string>::iterator &argP1,
                           std::list<std::string>::iterator &argP2)
@@ -233,7 +372,7 @@ namespace
       {
       *arg = "1";
       }
-    else 
+    else
       {
       *arg = "0";
       }
@@ -255,7 +394,7 @@ namespace
       {
       *arg = "1";
       }
-    else 
+    else
       {
       *arg = "0";
       }
@@ -298,7 +437,8 @@ namespace
   // level 0 processes parenthetical expressions
   bool HandleLevel0(std::list<std::string> &newArgs,
                     cmMakefile *makefile,
-                    std::string &errorString)
+                    std::string &errorString,
+                    cmake::MessageType &status)
   {
   int reducible;
   do
@@ -329,6 +469,7 @@ namespace
         if (depth)
           {
           errorString = "mismatched parenthesis in condition";
+          status = cmake::FATAL_ERROR;
           return false;
           }
         // store the reduced args in this vector
@@ -338,26 +479,26 @@ namespace
         std::list<std::string>::iterator argP1 = arg;
         argP1++;
         for(; argP1 != argClose; argP1++)
-          {   
+          {
           newArgs2.push_back(*argP1);
           }
         newArgs2.pop_back();
         // now recursively invoke IsTrue to handle the values inside the
         // parenthetical expression
-        bool value = 
-          cmIfCommand::IsTrue(newArgs2, errorString, makefile);
+        bool value =
+          cmIfCommand::IsTrue(newArgs2, errorString, makefile, status);
         if(value)
           {
           *arg = "1";
           }
-        else 
+        else
           {
           *arg = "0";
           }
         argP1 = arg;
         argP1++;
         // remove the now evaluated parenthetical expression
-        newArgs.erase(argP1,argClose);        
+        newArgs.erase(argP1,argClose);
         }
       ++arg;
       }
@@ -365,12 +506,12 @@ namespace
   while (reducible);
   return true;
   }
-  
+
   //=========================================================================
   // level one handles most predicates except for NOT
   bool HandleLevel1(std::list<std::string> &newArgs,
                     cmMakefile *makefile,
-                    std::string &)
+                    std::string &, cmake::MessageType &)
   {
   int reducible;
   do
@@ -397,6 +538,13 @@ namespace
           cmSystemTools::FileIsDirectory((argP1)->c_str()),
           reducible, arg, newArgs, argP1, argP2);
         }
+      // does a symlink with this name exist
+      if (*arg == "IS_SYMLINK" && argP1  != newArgs.end())
+        {
+        HandlePredicate(
+          cmSystemTools::FileIsSymlink((argP1)->c_str()),
+          reducible, arg, newArgs, argP1, argP2);
+        }
       // is the given path an absolute path ?
       if (*arg == "IS_ABSOLUTE" && argP1  != newArgs.end())
         {
@@ -454,7 +602,8 @@ namespace
   // level two handles most binary operations except for AND  OR
   bool HandleLevel2(std::list<std::string> &newArgs,
                     cmMakefile *makefile,
-                    std::string &errorString)
+                    std::string &errorString,
+                    cmake::MessageType &status)
   {
   int reducible;
   const char *def;
@@ -470,7 +619,7 @@ namespace
       argP1 = arg;
       IncrementArguments(newArgs,argP1,argP2);
       if (argP1 != newArgs.end() && argP2 != newArgs.end() &&
-        *(argP1) == "MATCHES") 
+        *(argP1) == "MATCHES")
         {
         def = cmIfCommand::GetVariableOrString(arg->c_str(), makefile);
         const char* rex = (argP2)->c_str();
@@ -481,6 +630,7 @@ namespace
           cmOStringStream error;
           error << "Regular expression \"" << rex << "\" cannot compile";
           errorString = error.str();
+          status = cmake::FATAL_ERROR;
           return false;
           }
         if (regEntry.find(def))
@@ -499,7 +649,7 @@ namespace
         reducible = 1;
         }
 
-      if (argP1 != newArgs.end() && *arg == "MATCHES") 
+      if (argP1 != newArgs.end() && *arg == "MATCHES")
         {
         *arg = "0";
         newArgs.erase(argP1);
@@ -509,8 +659,8 @@ namespace
         }
 
       if (argP1 != newArgs.end() && argP2 != newArgs.end() &&
-        (*(argP1) == "LESS" || *(argP1) == "GREATER" || 
-         *(argP1) == "EQUAL")) 
+        (*(argP1) == "LESS" || *(argP1) == "GREATER" ||
+         *(argP1) == "EQUAL"))
         {
         def = cmIfCommand::GetVariableOrString(arg->c_str(), makefile);
         def2 = cmIfCommand::GetVariableOrString((argP2)->c_str(), makefile);
@@ -529,19 +679,19 @@ namespace
         else if (*(argP1) == "GREATER")
           {
           result = (lhs > rhs);
-          }          
+          }
         else
           {
           result = (lhs == rhs);
-          }          
+          }
         HandleBinaryOp(result,
           reducible, arg, newArgs, argP1, argP2);
         }
 
       if (argP1 != newArgs.end() && argP2 != newArgs.end() &&
-        (*(argP1) == "STRLESS" || 
-         *(argP1) == "STREQUAL" || 
-         *(argP1) == "STRGREATER")) 
+        (*(argP1) == "STRLESS" ||
+         *(argP1) == "STREQUAL" ||
+         *(argP1) == "STRGREATER"))
         {
         def = cmIfCommand::GetVariableOrString(arg->c_str(), makefile);
         def2 = cmIfCommand::GetVariableOrString((argP2)->c_str(), makefile);
@@ -607,10 +757,11 @@ namespace
   // level 3 handles NOT
   bool HandleLevel3(std::list<std::string> &newArgs,
                     cmMakefile *makefile,
-                    std::string &)
+                    std::string &errorString,
+                    cmPolicies::PolicyStatus Policy12Status,
+                    cmake::MessageType &status)
   {
   int reducible;
-  const char *def;
   do
     {
     reducible = 0;
@@ -623,9 +774,11 @@ namespace
       IncrementArguments(newArgs,argP1,argP2);
       if (argP1 != newArgs.end() && *arg == "NOT")
         {
-        def = cmIfCommand::GetVariableOrNumber((argP1)->c_str(), makefile);
-        HandlePredicate(cmSystemTools::IsOff(def),
-          reducible, arg, newArgs, argP1, argP2);
+        bool rhs = GetBooleanValueWithAutoDereference(*argP1, makefile,
+                                                      errorString,
+                                                      Policy12Status,
+                                                      status);
+        HandlePredicate(!rhs, reducible, arg, newArgs, argP1, argP2);
         }
       ++arg;
       }
@@ -638,11 +791,13 @@ namespace
   // level 4 handles AND OR
   bool HandleLevel4(std::list<std::string> &newArgs,
                     cmMakefile *makefile,
-                    std::string &)
+                    std::string &errorString,
+                    cmPolicies::PolicyStatus Policy12Status,
+                    cmake::MessageType &status)
   {
   int reducible;
-  const char *def;
-  const char *def2;
+  bool lhs;
+  bool rhs;
   do
     {
     reducible = 0;
@@ -653,23 +808,33 @@ namespace
       {
       argP1 = arg;
       IncrementArguments(newArgs,argP1,argP2);
-      if (argP1 != newArgs.end() && *(argP1) == "AND" && 
+      if (argP1 != newArgs.end() && *(argP1) == "AND" &&
         argP2 != newArgs.end())
         {
-        def = cmIfCommand::GetVariableOrNumber(arg->c_str(), makefile);
-        def2 = cmIfCommand::GetVariableOrNumber((argP2)->c_str(), makefile);
-        HandleBinaryOp(
-          !(cmSystemTools::IsOff(def) || cmSystemTools::IsOff(def2)),
+        lhs = GetBooleanValueWithAutoDereference(*arg, makefile,
+                                                 errorString,
+                                                 Policy12Status,
+                                                 status);
+        rhs = GetBooleanValueWithAutoDereference(*argP2, makefile,
+                                                 errorString,
+                                                 Policy12Status,
+                                                 status);
+        HandleBinaryOp((lhs && rhs),
           reducible, arg, newArgs, argP1, argP2);
         }
 
-      if (argP1 != newArgs.end() && *(argP1) == "OR" && 
+      if (argP1 != newArgs.end() && *(argP1) == "OR" &&
         argP2 != newArgs.end())
         {
-        def = cmIfCommand::GetVariableOrNumber(arg->c_str(), makefile);
-        def2 = cmIfCommand::GetVariableOrNumber((argP2)->c_str(), makefile);
-        HandleBinaryOp(
-          !(cmSystemTools::IsOff(def) && cmSystemTools::IsOff(def2)),
+        lhs = GetBooleanValueWithAutoDereference(*arg, makefile,
+                                                 errorString,
+                                                 Policy12Status,
+                                                 status);
+        rhs = GetBooleanValueWithAutoDereference(*argP2, makefile,
+                                                 errorString,
+                                                 Policy12Status,
+                                                 status);
+        HandleBinaryOp((lhs || rhs),
           reducible, arg, newArgs, argP1, argP2);
         }
       ++arg;
@@ -683,9 +848,9 @@ namespace
 
 //=========================================================================
 // order of operations,
-// 1.   ( )   -- parenthetical groups 
-// 2.  IS_DIRECTORY EXISTS COMMAND DEFINED etc predicates 
-// 3. MATCHES LESS GREATER EQUAL STRLESS STRGREATER STREQUAL etc binary ops 
+// 1.   ( )   -- parenthetical groups
+// 2.  IS_DIRECTORY EXISTS COMMAND DEFINED etc predicates
+// 3. MATCHES LESS GREATER EQUAL STRLESS STRGREATER STREQUAL etc binary ops
 // 4. NOT
 // 5. AND OR
 //
@@ -699,9 +864,9 @@ namespace
 
 
 bool cmIfCommand::IsTrue(const std::vector<std::string> &args,
-                         std::string &errorString, cmMakefile *makefile)
+                         std::string &errorString, cmMakefile *makefile,
+                         cmake::MessageType &status)
 {
-  const char *def;
   errorString = "";
 
   // handle empty invocation
@@ -715,57 +880,58 @@ bool cmIfCommand::IsTrue(const std::vector<std::string> &args,
 
   // copy to the list structure
   for(unsigned int i = 0; i < args.size(); ++i)
-    {   
+    {
     newArgs.push_back(args[i]);
     }
 
   // now loop through the arguments and see if we can reduce any of them
   // we do this multiple times. Once for each level of precedence
-  if (!HandleLevel0(newArgs, makefile, errorString)) // parens
+  // parens
+  if (!HandleLevel0(newArgs, makefile, errorString, status))
     {
     return false;
     }
-  if (!HandleLevel1(newArgs, makefile, errorString)) //predicates
+  //predicates
+  if (!HandleLevel1(newArgs, makefile, errorString, status))
     {
     return false;
     }
-  if (!HandleLevel2(newArgs, makefile, errorString)) // binary ops
+  // binary ops
+  if (!HandleLevel2(newArgs, makefile, errorString, status))
     {
     return false;
     }
-  if (!HandleLevel3(newArgs, makefile, errorString)) // NOT
+
+  // used to store the value of policy CMP0012 for performance
+  cmPolicies::PolicyStatus Policy12Status =
+    makefile->GetPolicyStatus(cmPolicies::CMP0012);
+
+  // NOT
+  if (!HandleLevel3(newArgs, makefile, errorString,
+                    Policy12Status, status))
     {
     return false;
     }
-  if (!HandleLevel4(newArgs, makefile, errorString)) // AND OR
+  // AND OR
+  if (!HandleLevel4(newArgs, makefile, errorString,
+                    Policy12Status, status))
     {
     return false;
     }
 
   // now at the end there should only be one argument left
-  if (newArgs.size() == 1)
-    {
-    if (*newArgs.begin() == "0")
-      {
-      return false;
-      }
-    if (*newArgs.begin() == "1")
-      {
-      return true;
-      }
-    def = makefile->GetDefinition(args[0].c_str());
-    if(cmSystemTools::IsOff(def))
-      {
-      return false;
-      }
-    }
-  else
+  if (newArgs.size() != 1)
     {
     errorString = "Unknown arguments specified";
+    status = cmake::FATAL_ERROR;
     return false;
     }
-    
-  return true;
+
+  return GetBooleanValueWithAutoDereference(*(newArgs.begin()),
+                                            makefile,
+                                            errorString,
+                                            Policy12Status,
+                                            status, true);
 }
 
 //=========================================================================
@@ -779,18 +945,3 @@ const char* cmIfCommand::GetVariableOrString(const char* str,
     }
   return def;
 }
-
-//=========================================================================
-const char* cmIfCommand::GetVariableOrNumber(const char* str,
-                                             const cmMakefile* mf)
-{
-  const char* def = mf->GetDefinition(str);
-  if(!def)
-    {
-    if (atoi(str))
-      {
-      def = str;
-      }
-    }
-  return def;
-}
