@@ -1,19 +1,14 @@
-/*=========================================================================
+/*============================================================================
+  CMake - Cross Platform Makefile Generator
+  Copyright 2000-2009 Kitware, Inc., Insight Software Consortium
 
-  Program:   CMake - Cross-Platform Makefile Generator
-  Module:    $RCSfile: cmInstallCommand.cxx,v $
-  Language:  C++
-  Date:      $Date: 2008-04-23 16:14:33 $
-  Version:   $Revision: 1.45.2.2 $
+  Distributed under the OSI-approved BSD License (the "License");
+  see accompanying file Copyright.txt for details.
 
-  Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
-  See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notices for more information.
-
-=========================================================================*/
+  This software is distributed WITHOUT ANY WARRANTY; without even the
+  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+  See the License for more information.
+============================================================================*/
 #include "cmInstallCommand.h"
 
 #include "cmInstallDirectoryGenerator.h"
@@ -28,25 +23,25 @@
 static cmInstallTargetGenerator* CreateInstallTargetGenerator(cmTarget& target,
      const cmInstallCommandArguments& args, bool impLib, bool forceOpt = false)
 {
-  return new cmInstallTargetGenerator(target, args.GetDestination().c_str(), 
-                        impLib, args.GetPermissions().c_str(), 
+  return new cmInstallTargetGenerator(target, args.GetDestination().c_str(),
+                        impLib, args.GetPermissions().c_str(),
                         args.GetConfigurations(), args.GetComponent().c_str(),
                         args.GetOptional() || forceOpt);
 }
 
 static cmInstallFilesGenerator* CreateInstallFilesGenerator(
-    const std::vector<std::string>& absFiles, 
+    const std::vector<std::string>& absFiles,
     const cmInstallCommandArguments& args, bool programs)
 {
-  return new cmInstallFilesGenerator(absFiles, args.GetDestination().c_str(), 
-                        programs, args.GetPermissions().c_str(), 
+  return new cmInstallFilesGenerator(absFiles, args.GetDestination().c_str(),
+                        programs, args.GetPermissions().c_str(),
                         args.GetConfigurations(), args.GetComponent().c_str(),
                         args.GetRename().c_str(), args.GetOptional());
 }
 
 
 // cmInstallCommand
-bool cmInstallCommand::InitialPass(std::vector<std::string> const& args, 
+bool cmInstallCommand::InitialPass(std::vector<std::string> const& args,
                                    cmExecutionStatus &)
 {
   // Allow calling with no arguments so that arguments may be built up
@@ -60,6 +55,13 @@ bool cmInstallCommand::InitialPass(std::vector<std::string> const& args,
   this->Makefile->GetLocalGenerator()
     ->GetGlobalGenerator()->EnableInstallTarget();
 
+  this->DefaultComponentName = this->Makefile->GetSafeDefinition(
+                                       "CMAKE_INSTALL_DEFAULT_COMPONENT_NAME");
+  if (this->DefaultComponentName.empty())
+    {
+    this->DefaultComponentName = "Unspecified";
+    }
+
   // Switch among the command modes.
   if(args[0] == "SCRIPT")
     {
@@ -100,7 +102,7 @@ bool cmInstallCommand::InitialPass(std::vector<std::string> const& args,
 //----------------------------------------------------------------------------
 bool cmInstallCommand::HandleScriptMode(std::vector<std::string> const& args)
 {
-  std::string component("Unspecified");
+  std::string component = this->DefaultComponentName;
   int componentCount = 0;
   bool doing_script = false;
   bool doing_code = false;
@@ -195,7 +197,7 @@ bool cmInstallCommand::HandleScriptMode(std::vector<std::string> const& args)
 
 /*struct InstallPart
 {
-  InstallPart(cmCommandArgumentsHelper* helper, const char* key, 
+  InstallPart(cmCommandArgumentsHelper* helper, const char* key,
          cmCommandArgumentGroup* group);
   cmCAStringVector argVector;
   cmInstallCommandArguments args;
@@ -227,7 +229,7 @@ bool cmInstallCommand::HandleTargetsMode(std::vector<std::string> const& args)
   // ARCHIVE, RUNTIME etc. (see above)
   // These generic args also contain the targets and the export stuff
   std::vector<std::string> unknownArgs;
-  cmInstallCommandArguments genericArgs;
+  cmInstallCommandArguments genericArgs(this->DefaultComponentName);
   cmCAStringVector targetList(&genericArgs.Parser, "TARGETS");
   cmCAString exports(&genericArgs.Parser,"EXPORT", &genericArgs.ArgumentGroup);
   targetList.Follows(0);
@@ -235,16 +237,16 @@ bool cmInstallCommand::HandleTargetsMode(std::vector<std::string> const& args)
   genericArgs.Parse(&genericArgVector.GetVector(), &unknownArgs);
   bool success = genericArgs.Finalize();
 
-  cmInstallCommandArguments archiveArgs;
-  cmInstallCommandArguments libraryArgs;
-  cmInstallCommandArguments runtimeArgs;
-  cmInstallCommandArguments frameworkArgs;
-  cmInstallCommandArguments bundleArgs;
-  cmInstallCommandArguments privateHeaderArgs;
-  cmInstallCommandArguments publicHeaderArgs;
-  cmInstallCommandArguments resourceArgs;
+  cmInstallCommandArguments archiveArgs(this->DefaultComponentName);
+  cmInstallCommandArguments libraryArgs(this->DefaultComponentName);
+  cmInstallCommandArguments runtimeArgs(this->DefaultComponentName);
+  cmInstallCommandArguments frameworkArgs(this->DefaultComponentName);
+  cmInstallCommandArguments bundleArgs(this->DefaultComponentName);
+  cmInstallCommandArguments privateHeaderArgs(this->DefaultComponentName);
+  cmInstallCommandArguments publicHeaderArgs(this->DefaultComponentName);
+  cmInstallCommandArguments resourceArgs(this->DefaultComponentName);
 
-  // now parse the args for specific parts of the target (e.g. LIBRARY, 
+  // now parse the args for specific parts of the target (e.g. LIBRARY,
   // RUNTIME, ARCHIVE etc.
   archiveArgs.Parse      (&archiveArgVector.GetVector(),       &unknownArgs);
   libraryArgs.Parse      (&libraryArgVector.GetVector(),       &unknownArgs);
@@ -350,40 +352,60 @@ bool cmInstallCommand::HandleTargetsMode(std::vector<std::string> const& args)
                        this->Makefile->IsOn("CYGWIN") ||
                        this->Makefile->IsOn("MINGW"));
 
-  for(std::vector<std::string>::const_iterator 
+  for(std::vector<std::string>::const_iterator
       targetIt=targetList.GetVector().begin();
       targetIt!=targetList.GetVector().end();
       ++targetIt)
     {
-      // Lookup this target in the current directory.
-      if(cmTarget* target=this->Makefile->FindTarget(targetIt->c_str()))
+    // Lookup this target in the current directory.
+    if(cmTarget* target=this->Makefile->FindTarget(targetIt->c_str()))
+      {
+      // Found the target.  Check its type.
+      if(target->GetType() != cmTarget::EXECUTABLE &&
+         target->GetType() != cmTarget::STATIC_LIBRARY &&
+         target->GetType() != cmTarget::SHARED_LIBRARY &&
+         target->GetType() != cmTarget::MODULE_LIBRARY &&
+         target->GetType() != cmTarget::OBJECT_LIBRARY)
         {
-        // Found the target.  Check its type.
-        if(target->GetType() != cmTarget::EXECUTABLE &&
-           target->GetType() != cmTarget::STATIC_LIBRARY &&
-           target->GetType() != cmTarget::SHARED_LIBRARY &&
-           target->GetType() != cmTarget::MODULE_LIBRARY)
-          {
-          cmOStringStream e;
-          e << "TARGETS given target \"" << (*targetIt)
-            << "\" which is not an executable, library, or module.";
-          this->SetError(e.str().c_str());
-          return false;
-          }
-        // Store the target in the list to be installed.
-        targets.push_back(target);
-        }
-      else
-        {
-        // Did not find the target.
         cmOStringStream e;
         e << "TARGETS given target \"" << (*targetIt)
-          << "\" which does not exist in this directory.";
+          << "\" which is not an executable, library, or module.";
         this->SetError(e.str().c_str());
         return false;
         }
+      else if(target->GetType() == cmTarget::OBJECT_LIBRARY)
+        {
+        cmOStringStream e;
+        e << "TARGETS given OBJECT library \"" << (*targetIt)
+          << "\" which may not be installed.";
+        this->SetError(e.str().c_str());
+        return false;
+        }
+      // Store the target in the list to be installed.
+      targets.push_back(target);
+      }
+    else
+      {
+      // Did not find the target.
+      cmOStringStream e;
+      e << "TARGETS given target \"" << (*targetIt)
+        << "\" which does not exist in this directory.";
+      this->SetError(e.str().c_str());
+      return false;
+      }
     }
 
+  // Keep track of whether we will be performing an installation of
+  // any files of the given type.
+  bool installsArchive = false;
+  bool installsLibrary = false;
+  bool installsRuntime = false;
+  bool installsFramework = false;
+  bool installsBundle = false;
+  bool installsPrivateHeader = false;
+  bool installsPublicHeader = false;
+  bool installsResource = false;
+
   // Generate install script code to install the given targets.
   for(std::vector<cmTarget*>::iterator ti = targets.begin();
       ti != targets.end(); ++ti)
@@ -407,7 +429,7 @@ bool cmInstallCommand::HandleTargetsMode(std::vector<std::string> const& args)
       case cmTarget::SHARED_LIBRARY:
         {
         // Shared libraries are handled differently on DLL and non-DLL
-        // platforms.  All windows platforms are DLL platforms including 
+        // platforms.  All windows platforms are DLL platforms including
         // cygwin.  Currently no other platform is a DLL platform.
         if(dll_platform)
           {
@@ -421,13 +443,13 @@ bool cmInstallCommand::HandleTargetsMode(std::vector<std::string> const& args)
           if(!archiveArgs.GetDestination().empty())
             {
             // The import library uses the ARCHIVE properties.
-            archiveGenerator = CreateInstallTargetGenerator(target, 
+            archiveGenerator = CreateInstallTargetGenerator(target,
                                                             archiveArgs, true);
             }
           if(!runtimeArgs.GetDestination().empty())
             {
             // The DLL uses the RUNTIME properties.
-            runtimeGenerator = CreateInstallTargetGenerator(target, 
+            runtimeGenerator = CreateInstallTargetGenerator(target,
                                                            runtimeArgs, false);
             }
           if ((archiveGenerator==0) && (runtimeGenerator==0))
@@ -452,7 +474,7 @@ bool cmInstallCommand::HandleTargetsMode(std::vector<std::string> const& args)
             // Use the FRAMEWORK properties.
             if (!frameworkArgs.GetDestination().empty())
               {
-              frameworkGenerator = CreateInstallTargetGenerator(target, 
+              frameworkGenerator = CreateInstallTargetGenerator(target,
                                                          frameworkArgs, false);
               }
             else
@@ -469,7 +491,7 @@ bool cmInstallCommand::HandleTargetsMode(std::vector<std::string> const& args)
             // The shared library uses the LIBRARY properties.
             if (!libraryArgs.GetDestination().empty())
               {
-              libraryGenerator = CreateInstallTargetGenerator(target, 
+              libraryGenerator = CreateInstallTargetGenerator(target,
                                                            libraryArgs, false);
               libraryGenerator->SetNamelinkMode(namelinkMode);
               namelinkOnly =
@@ -492,7 +514,7 @@ bool cmInstallCommand::HandleTargetsMode(std::vector<std::string> const& args)
         // Static libraries use ARCHIVE properties.
         if (!archiveArgs.GetDestination().empty())
           {
-          archiveGenerator = CreateInstallTargetGenerator(target, archiveArgs, 
+          archiveGenerator = CreateInstallTargetGenerator(target, archiveArgs,
                                                           false);
           }
         else
@@ -510,7 +532,7 @@ bool cmInstallCommand::HandleTargetsMode(std::vector<std::string> const& args)
         // Modules use LIBRARY properties.
         if (!libraryArgs.GetDestination().empty())
           {
-          libraryGenerator = CreateInstallTargetGenerator(target, libraryArgs, 
+          libraryGenerator = CreateInstallTargetGenerator(target, libraryArgs,
                                                           false);
           libraryGenerator->SetNamelinkMode(namelinkMode);
           namelinkOnly =
@@ -533,7 +555,7 @@ bool cmInstallCommand::HandleTargetsMode(std::vector<std::string> const& args)
           // Application bundles use the BUNDLE properties.
           if (!bundleArgs.GetDestination().empty())
             {
-            bundleGenerator = CreateInstallTargetGenerator(target, bundleArgs, 
+            bundleGenerator = CreateInstallTargetGenerator(target, bundleArgs,
                                                            false);
             }
           else if(!runtimeArgs.GetDestination().empty())
@@ -565,7 +587,7 @@ bool cmInstallCommand::HandleTargetsMode(std::vector<std::string> const& args)
           // Executables use the RUNTIME properties.
           if (!runtimeArgs.GetDestination().empty())
             {
-            runtimeGenerator = CreateInstallTargetGenerator(target, 
+            runtimeGenerator = CreateInstallTargetGenerator(target,
                                                            runtimeArgs, false);
             }
           else
@@ -585,7 +607,7 @@ bool cmInstallCommand::HandleTargetsMode(std::vector<std::string> const& args)
            target.IsExecutableWithExports())
           {
           // The import library uses the ARCHIVE properties.
-          archiveGenerator = CreateInstallTargetGenerator(target, 
+          archiveGenerator = CreateInstallTargetGenerator(target,
                                                       archiveArgs, true, true);
           }
         }
@@ -596,100 +618,109 @@ bool cmInstallCommand::HandleTargetsMode(std::vector<std::string> const& args)
         break;
       }
 
-//       if(target.GetProperty("ASSOCIATED_FILES");
+    // These well-known sets of files are installed *automatically* for
+    // FRAMEWORK SHARED library targets on the Mac as part of installing the
+    // FRAMEWORK.  For other target types or on other platforms, they are not
+    // installed automatically and so we need to create install files
+    // generators for them.
+    bool createInstallGeneratorsForTargetFileSets = true;
 
-  // These well-known sets of files are installed *automatically* for FRAMEWORK
-  // SHARED library targets on the Mac as part of installing the FRAMEWORK.
-  // For other target types or on other platforms, they are not installed
-  // automatically and so we need to create install files generators for them.
-  //
-  bool createInstallGeneratorsForTargetFileSets = true;
-
-  if(target.IsFrameworkOnApple())
-    {
-    createInstallGeneratorsForTargetFileSets = false;
-    }
-
-  if(createInstallGeneratorsForTargetFileSets && !namelinkOnly)
-    {
-    const char* files = target.GetProperty("PRIVATE_HEADER");
-    if ((files) && (*files))
+    if(target.IsFrameworkOnApple())
       {
-      std::vector<std::string> relFiles;
-      cmSystemTools::ExpandListArgument(files, relFiles);
-      std::vector<std::string> absFiles;
-      if (!this->MakeFilesFullPath("PRIVATE_HEADER", relFiles, absFiles))
+      createInstallGeneratorsForTargetFileSets = false;
+      }
+
+    if(createInstallGeneratorsForTargetFileSets && !namelinkOnly)
+      {
+      const char* files = target.GetProperty("PRIVATE_HEADER");
+      if ((files) && (*files))
         {
-        return false;
+        std::vector<std::string> relFiles;
+        cmSystemTools::ExpandListArgument(files, relFiles);
+        std::vector<std::string> absFiles;
+        if (!this->MakeFilesFullPath("PRIVATE_HEADER", relFiles, absFiles))
+          {
+          return false;
+          }
+
+        // Create the files install generator.
+        if (!privateHeaderArgs.GetDestination().empty())
+          {
+          privateHeaderGenerator =
+            CreateInstallFilesGenerator(absFiles, privateHeaderArgs, false);
+          }
+        else
+          {
+          cmOStringStream e;
+          e << "INSTALL TARGETS - target " << target.GetName() << " has "
+            << "PRIVATE_HEADER files but no PRIVATE_HEADER DESTINATION.";
+          cmSystemTools::Message(e.str().c_str(), "Warning");
+          }
         }
 
-      // Create the files install generator.
-      if (!privateHeaderArgs.GetDestination().empty())
+      files = target.GetProperty("PUBLIC_HEADER");
+      if ((files) && (*files))
         {
-        privateHeaderGenerator = CreateInstallFilesGenerator(absFiles, 
-                                                     privateHeaderArgs, false);
+        std::vector<std::string> relFiles;
+        cmSystemTools::ExpandListArgument(files, relFiles);
+        std::vector<std::string> absFiles;
+        if (!this->MakeFilesFullPath("PUBLIC_HEADER", relFiles, absFiles))
+          {
+          return false;
+          }
+
+        // Create the files install generator.
+        if (!publicHeaderArgs.GetDestination().empty())
+          {
+          publicHeaderGenerator =
+            CreateInstallFilesGenerator(absFiles, publicHeaderArgs, false);
+          }
+        else
+          {
+          cmOStringStream e;
+          e << "INSTALL TARGETS - target " << target.GetName() << " has "
+            << "PUBLIC_HEADER files but no PUBLIC_HEADER DESTINATION.";
+          cmSystemTools::Message(e.str().c_str(), "Warning");
+          }
         }
-      else
+
+      files = target.GetProperty("RESOURCE");
+      if ((files) && (*files))
         {
-        cmOStringStream e;
-        e << "INSTALL TARGETS - target " << target.GetName() << " has "
-          << "PRIVATE_HEADER files but no PRIVATE_HEADER DESTINATION.";
-        cmSystemTools::Message(e.str().c_str(), "Warning");
+        std::vector<std::string> relFiles;
+        cmSystemTools::ExpandListArgument(files, relFiles);
+        std::vector<std::string> absFiles;
+        if (!this->MakeFilesFullPath("RESOURCE", relFiles, absFiles))
+          {
+          return false;
+          }
+
+        // Create the files install generator.
+        if (!resourceArgs.GetDestination().empty())
+          {
+          resourceGenerator = CreateInstallFilesGenerator(absFiles,
+                                                          resourceArgs, false);
+          }
+        else
+          {
+          cmOStringStream e;
+          e << "INSTALL TARGETS - target " << target.GetName() << " has "
+            << "RESOURCE files but no RESOURCE DESTINATION.";
+          cmSystemTools::Message(e.str().c_str(), "Warning");
+          }
         }
       }
 
-    files = target.GetProperty("PUBLIC_HEADER");
-    if ((files) && (*files))
-      {
-      std::vector<std::string> relFiles;
-      cmSystemTools::ExpandListArgument(files, relFiles);
-      std::vector<std::string> absFiles;
-      if (!this->MakeFilesFullPath("PUBLIC_HEADER", relFiles, absFiles))
-        {
-        return false;
-        }
-
-      // Create the files install generator.
-      if (!publicHeaderArgs.GetDestination().empty())
-        {
-        publicHeaderGenerator = CreateInstallFilesGenerator(absFiles, 
-                                                      publicHeaderArgs, false);
-        }
-      else
-        {
-        cmOStringStream e;
-        e << "INSTALL TARGETS - target " << target.GetName() << " has "
-          << "PUBLIC_HEADER files but no PUBLIC_HEADER DESTINATION.";
-        cmSystemTools::Message(e.str().c_str(), "Warning");
-        }
-      }
-
-    files = target.GetProperty("RESOURCE");
-    if ((files) && (*files))
-      {
-      std::vector<std::string> relFiles;
-      cmSystemTools::ExpandListArgument(files, relFiles);
-      std::vector<std::string> absFiles;
-      if (!this->MakeFilesFullPath("RESOURCE", relFiles, absFiles))
-        {
-        return false;
-        }
-
-      // Create the files install generator.
-      if (!resourceArgs.GetDestination().empty())
-        {
-        resourceGenerator = CreateInstallFilesGenerator(absFiles, 
-                                                        resourceArgs, false);
-        }
-      else
-        {
-        cmOStringStream e;
-        e << "INSTALL TARGETS - target " << target.GetName() << " has "
-          << "RESOURCE files but no RESOURCE DESTINATION.";
-        cmSystemTools::Message(e.str().c_str(), "Warning");
-        }
-      }
-    }
+    // Keep track of whether we're installing anything in each category
+    installsArchive = installsArchive || archiveGenerator != 0;
+    installsLibrary = installsLibrary || libraryGenerator != 0;
+    installsRuntime = installsRuntime || runtimeGenerator != 0;
+    installsFramework = installsFramework || frameworkGenerator != 0;
+    installsBundle = installsBundle || bundleGenerator != 0;
+    installsPrivateHeader = installsPrivateHeader
+      || privateHeaderGenerator != 0;
+    installsPublicHeader = installsPublicHeader || publicHeaderGenerator != 0;
+    installsResource = installsResource || resourceGenerator;
 
     this->Makefile->AddInstallGenerator(archiveGenerator);
     this->Makefile->AddInstallGenerator(libraryGenerator);
@@ -705,30 +736,56 @@ bool cmInstallCommand::HandleTargetsMode(std::vector<std::string> const& args)
     if(!exports.GetString().empty() && !namelinkOnly)
       {
       this->Makefile->GetLocalGenerator()->GetGlobalGenerator()
-        ->AddTargetToExports(exports.GetCString(), &target, 
+        ->AddTargetToExports(exports.GetCString(), &target,
                              archiveGenerator, runtimeGenerator,
                              libraryGenerator, frameworkGenerator,
                              bundleGenerator, publicHeaderGenerator);
       }
     }
 
-  // Tell the global generator about any installation component names specified
-  this->Makefile->GetLocalGenerator()->GetGlobalGenerator()
-                     ->AddInstallComponent(archiveArgs.GetComponent().c_str());
-  this->Makefile->GetLocalGenerator()->GetGlobalGenerator()
-                     ->AddInstallComponent(libraryArgs.GetComponent().c_str());
-  this->Makefile->GetLocalGenerator()->GetGlobalGenerator()
-                     ->AddInstallComponent(runtimeArgs.GetComponent().c_str());
-  this->Makefile->GetLocalGenerator()->GetGlobalGenerator()
-                   ->AddInstallComponent(frameworkArgs.GetComponent().c_str());
-  this->Makefile->GetLocalGenerator()->GetGlobalGenerator()
-                      ->AddInstallComponent(bundleArgs.GetComponent().c_str());
-  this->Makefile->GetLocalGenerator()->GetGlobalGenerator()
-               ->AddInstallComponent(privateHeaderArgs.GetComponent().c_str());
-  this->Makefile->GetLocalGenerator()->GetGlobalGenerator()
-                ->AddInstallComponent(publicHeaderArgs.GetComponent().c_str());
-  this->Makefile->GetLocalGenerator()->GetGlobalGenerator()
-                    ->AddInstallComponent(resourceArgs.GetComponent().c_str());
+  // Tell the global generator about any installation component names
+  // specified
+  if (installsArchive)
+    {
+    this->Makefile->GetLocalGenerator()->
+      GetGlobalGenerator()
+      ->AddInstallComponent(archiveArgs.GetComponent().c_str());
+    }
+  if (installsLibrary)
+    {
+    this->Makefile->GetLocalGenerator()->GetGlobalGenerator()
+      ->AddInstallComponent(libraryArgs.GetComponent().c_str());
+    }
+  if (installsRuntime)
+    {
+    this->Makefile->GetLocalGenerator()->GetGlobalGenerator()
+      ->AddInstallComponent(runtimeArgs.GetComponent().c_str());
+    }
+  if (installsFramework)
+    {
+    this->Makefile->GetLocalGenerator()->GetGlobalGenerator()
+      ->AddInstallComponent(frameworkArgs.GetComponent().c_str());
+    }
+  if (installsBundle)
+    {
+    this->Makefile->GetLocalGenerator()->GetGlobalGenerator()
+      ->AddInstallComponent(bundleArgs.GetComponent().c_str());
+    }
+  if (installsPrivateHeader)
+    {
+    this->Makefile->GetLocalGenerator()->GetGlobalGenerator()
+      ->AddInstallComponent(privateHeaderArgs.GetComponent().c_str());
+    }
+  if (installsPublicHeader)
+    {
+    this->Makefile->GetLocalGenerator()->GetGlobalGenerator()
+      ->AddInstallComponent(publicHeaderArgs.GetComponent().c_str());
+    }
+  if (installsResource)
+    {
+    this->Makefile->GetLocalGenerator()->GetGlobalGenerator()
+      ->AddInstallComponent(resourceArgs.GetComponent().c_str());
+    }
 
   return true;
 }
@@ -738,7 +795,7 @@ bool cmInstallCommand::HandleFilesMode(std::vector<std::string> const& args)
 {
   // This is the FILES mode.
   bool programs = (args[0] == "PROGRAMS");
-  cmInstallCommandArguments ica;
+  cmInstallCommandArguments ica(this->DefaultComponentName);
   cmCAStringVector files(&ica.Parser, programs ? "PROGRAMS" : "FILES");
   files.Follows(0);
   ica.ArgumentGroup.Follows(&files);
@@ -753,7 +810,7 @@ bool cmInstallCommand::HandleFilesMode(std::vector<std::string> const& args)
     this->SetError(e.str().c_str());
     return false;
     }
-  
+
   // Check if there is something to do.
   if(files.GetVector().empty())
     {
@@ -804,22 +861,18 @@ bool cmInstallCommand::HandleFilesMode(std::vector<std::string> const& args)
 bool
 cmInstallCommand::HandleDirectoryMode(std::vector<std::string> const& args)
 {
-  bool doing_dirs = true;
-  bool doing_destination = false;
-  bool doing_pattern = false;
-  bool doing_regex = false;
-  bool doing_permissions_file = false;
-  bool doing_permissions_dir = false;
-  bool doing_permissions_match = false;
-  bool doing_configurations = false;
-  bool doing_component = false;
+  enum Doing { DoingNone, DoingDirs, DoingDestination, DoingPattern,
+               DoingRegex, DoingPermsFile, DoingPermsDir, DoingPermsMatch,
+               DoingConfigurations, DoingComponent };
+  Doing doing = DoingDirs;
   bool in_match_mode = false;
+  bool optional = false;
   std::vector<std::string> dirs;
   const char* destination = 0;
   std::string permissions_file;
   std::string permissions_dir;
   std::vector<std::string> configurations;
-  std::string component = "Unspecified";
+  std::string component = this->DefaultComponentName;
   std::string literal_args;
   for(unsigned int i=1; i < args.size(); ++i)
     {
@@ -835,47 +888,39 @@ cmInstallCommand::HandleDirectoryMode(std::vector<std::string> const& args)
         }
 
       // Switch to setting the destination property.
-      doing_dirs = false;
-      doing_destination = true;
-      doing_pattern = false;
-      doing_regex = false;
-      doing_permissions_file = false;
-      doing_permissions_dir = false;
-      doing_configurations = false;
-      doing_component = false;
+      doing = DoingDestination;
+      }
+    else if(args[i] == "OPTIONAL")
+      {
+      if(in_match_mode)
+        {
+        cmOStringStream e;
+        e << args[0] << " does not allow \""
+          << args[i] << "\" after PATTERN or REGEX.";
+        this->SetError(e.str().c_str());
+        return false;
+        }
+
+      // Mark the rule as optional.
+      optional = true;
+      doing = DoingNone;
       }
     else if(args[i] == "PATTERN")
       {
       // Switch to a new pattern match rule.
-      doing_dirs = false;
-      doing_destination = false;
-      doing_pattern = true;
-      doing_regex = false;
-      doing_permissions_file = false;
-      doing_permissions_dir = false;
-      doing_permissions_match = false;
-      doing_configurations = false;
-      doing_component = false;
+      doing = DoingPattern;
       in_match_mode = true;
       }
     else if(args[i] == "REGEX")
       {
       // Switch to a new regex match rule.
-      doing_dirs = false;
-      doing_destination = false;
-      doing_pattern = false;
-      doing_regex = true;
-      doing_permissions_file = false;
-      doing_permissions_dir = false;
-      doing_permissions_match = false;
-      doing_configurations = false;
-      doing_component = false;
+      doing = DoingRegex;
       in_match_mode = true;
       }
     else if(args[i] == "EXCLUDE")
       {
       // Add this property to the current match rule.
-      if(!in_match_mode || doing_pattern || doing_regex)
+      if(!in_match_mode || doing == DoingPattern || doing == DoingRegex)
         {
         cmOStringStream e;
         e << args[0] << " does not allow \""
@@ -884,7 +929,7 @@ cmInstallCommand::HandleDirectoryMode(std::vector<std::string> const& args)
         return false;
         }
       literal_args += " EXCLUDE";
-      doing_permissions_match = false;
+      doing = DoingNone;
       }
     else if(args[i] == "PERMISSIONS")
       {
@@ -899,7 +944,7 @@ cmInstallCommand::HandleDirectoryMode(std::vector<std::string> const& args)
 
       // Switch to setting the current match permissions property.
       literal_args += " PERMISSIONS";
-      doing_permissions_match = true;
+      doing = DoingPermsMatch;
       }
     else if(args[i] == "FILE_PERMISSIONS")
       {
@@ -913,14 +958,7 @@ cmInstallCommand::HandleDirectoryMode(std::vector<std::string> const& args)
         }
 
       // Switch to setting the file permissions property.
-      doing_dirs = false;
-      doing_destination = false;
-      doing_pattern = false;
-      doing_regex = false;
-      doing_permissions_file = true;
-      doing_permissions_dir = false;
-      doing_configurations = false;
-      doing_component = false;
+      doing = DoingPermsFile;
       }
     else if(args[i] == "DIRECTORY_PERMISSIONS")
       {
@@ -934,14 +972,7 @@ cmInstallCommand::HandleDirectoryMode(std::vector<std::string> const& args)
         }
 
       // Switch to setting the directory permissions property.
-      doing_dirs = false;
-      doing_destination = false;
-      doing_pattern = false;
-      doing_regex = false;
-      doing_permissions_file = false;
-      doing_permissions_dir = true;
-      doing_configurations = false;
-      doing_component = false;
+      doing = DoingPermsDir;
       }
     else if(args[i] == "USE_SOURCE_PERMISSIONS")
       {
@@ -955,15 +986,8 @@ cmInstallCommand::HandleDirectoryMode(std::vector<std::string> const& args)
         }
 
       // Add this option literally.
-      doing_dirs = false;
-      doing_destination = false;
-      doing_pattern = false;
-      doing_regex = false;
-      doing_permissions_file = false;
-      doing_permissions_dir = false;
-      doing_configurations = false;
-      doing_component = false;
       literal_args += " USE_SOURCE_PERMISSIONS";
+      doing = DoingNone;
       }
     else if(args[i] == "FILES_MATCHING")
       {
@@ -977,16 +1001,8 @@ cmInstallCommand::HandleDirectoryMode(std::vector<std::string> const& args)
         }
 
       // Add this option literally.
-      doing_dirs = false;
-      doing_destination = false;
-      doing_pattern = false;
-      doing_regex = false;
-      doing_permissions_file = false;
-      doing_permissions_dir = false;
-      doing_permissions_match = false;
-      doing_configurations = false;
-      doing_component = false;
       literal_args += " FILES_MATCHING";
+      doing = DoingNone;
       }
     else if(args[i] == "CONFIGURATIONS")
       {
@@ -1000,14 +1016,7 @@ cmInstallCommand::HandleDirectoryMode(std::vector<std::string> const& args)
         }
 
       // Switch to setting the configurations property.
-      doing_dirs = false;
-      doing_destination = false;
-      doing_pattern = false;
-      doing_regex = false;
-      doing_permissions_file = false;
-      doing_permissions_dir = false;
-      doing_configurations = true;
-      doing_component = false;
+      doing = DoingConfigurations;
       }
     else if(args[i] == "COMPONENT")
       {
@@ -1021,16 +1030,9 @@ cmInstallCommand::HandleDirectoryMode(std::vector<std::string> const& args)
         }
 
       // Switch to setting the component property.
-      doing_dirs = false;
-      doing_destination = false;
-      doing_pattern = false;
-      doing_regex = false;
-      doing_permissions_file = false;
-      doing_permissions_dir = false;
-      doing_configurations = false;
-      doing_component = true;
+      doing = DoingComponent;
       }
-    else if(doing_dirs)
+    else if(doing == DoingDirs)
       {
       // Convert this directory to a full path.
       std::string dir = args[i];
@@ -1055,16 +1057,16 @@ cmInstallCommand::HandleDirectoryMode(std::vector<std::string> const& args)
       // Store the directory for installation.
       dirs.push_back(dir);
       }
-    else if(doing_configurations)
+    else if(doing == DoingConfigurations)
       {
       configurations.push_back(args[i]);
       }
-    else if(doing_destination)
+    else if(doing == DoingDestination)
       {
       destination = args[i].c_str();
-      doing_destination = false;
+      doing = DoingNone;
       }
-    else if(doing_pattern)
+    else if(doing == DoingPattern)
       {
       // Convert the pattern to a regular expression.  Require a
       // leading slash and trailing end-of-string in the matched
@@ -1075,9 +1077,9 @@ cmInstallCommand::HandleDirectoryMode(std::vector<std::string> const& args)
       cmSystemTools::ReplaceString(regex, "\\", "\\\\");
       literal_args += regex;
       literal_args += "$\"";
-      doing_pattern = false;
+      doing = DoingNone;
       }
-    else if(doing_regex)
+    else if(doing == DoingRegex)
       {
       literal_args += " REGEX \"";
     // Match rules are case-insensitive on some platforms.
@@ -1089,14 +1091,14 @@ cmInstallCommand::HandleDirectoryMode(std::vector<std::string> const& args)
       cmSystemTools::ReplaceString(regex, "\\", "\\\\");
       literal_args += regex;
       literal_args += "\"";
-      doing_regex = false;
+      doing = DoingNone;
       }
-    else if(doing_component)
+    else if(doing == DoingComponent)
       {
       component = args[i];
-      doing_component = false;
+      doing = DoingNone;
       }
-    else if(doing_permissions_file)
+    else if(doing == DoingPermsFile)
      {
      // Check the requested permission.
      if(!cmInstallCommandArguments::CheckPermissions(args[i],permissions_file))
@@ -1108,7 +1110,7 @@ cmInstallCommand::HandleDirectoryMode(std::vector<std::string> const& args)
         return false;
         }
       }
-    else if(doing_permissions_dir)
+    else if(doing == DoingPermsDir)
       {
       // Check the requested permission.
       if(!cmInstallCommandArguments::CheckPermissions(args[i],permissions_dir))
@@ -1120,7 +1122,7 @@ cmInstallCommand::HandleDirectoryMode(std::vector<std::string> const& args)
         return false;
         }
       }
-    else if(doing_permissions_match)
+    else if(doing == DoingPermsMatch)
       {
       // Check the requested permission.
       if(!cmInstallCommandArguments::CheckPermissions(args[i], literal_args))
@@ -1169,7 +1171,8 @@ cmInstallCommand::HandleDirectoryMode(std::vector<std::string> const& args)
                                     permissions_dir.c_str(),
                                     configurations,
                                     component.c_str(),
-                                    literal_args.c_str()));
+                                    literal_args.c_str(),
+                                    optional));
 
   // Tell the global generator about any installation component names
   // specified.
@@ -1183,7 +1186,7 @@ cmInstallCommand::HandleDirectoryMode(std::vector<std::string> const& args)
 bool cmInstallCommand::HandleExportMode(std::vector<std::string> const& args)
 {
   // This is the EXPORT mode.
-  cmInstallCommandArguments ica;
+  cmInstallCommandArguments ica(this->DefaultComponentName);
   cmCAString exp(&ica.Parser, "EXPORT");
   cmCAString name_space(&ica.Parser, "NAMESPACE", &ica.ArgumentGroup);
   cmCAString filename(&ica.Parser, "FILE", &ica.ArgumentGroup);
