@@ -41,12 +41,12 @@ public:
   /**
    * The name of the command as specified in CMakeList.txt.
    */
-  virtual const char* GetName() { return "install";}
+  virtual const char* GetName() const { return "install";}
 
   /**
    * Succinct documentation.
    */
-  virtual const char* GetTerseDocumentation()
+  virtual const char* GetTerseDocumentation() const
     {
     return "Specify rules to run at install time.";
     }
@@ -54,7 +54,7 @@ public:
   /**
    * More documentation.
    */
-  virtual const char* GetFullDocumentation()
+  virtual const char* GetFullDocumentation() const
     {
     return
       "This command generates installation rules for a project.  "
@@ -85,7 +85,10 @@ public:
       "with which the install rule is associated, such as \"runtime\" or "
       "\"development\".  During component-specific installation only "
       "install rules associated with the given component name will be "
-      "executed.  During a full installation all components are installed.\n"
+      "executed.  During a full installation all components are installed."
+      " If COMPONENT is not provided a default component \"Unspecified\" is"
+      " created. The default component name may be controlled with the "
+      "CMAKE_INSTALL_DEFAULT_COMPONENT_NAME variable.\n"
       "The RENAME argument specifies a name for an installed file that "
       "may be different from the original file.  Renaming is allowed only "
       "when a single file is installed by the command.\n"
@@ -168,13 +171,12 @@ public:
       "On non-DLL platforms mySharedLib will be installed to <prefix>/lib "
       "and /some/full/path.  On DLL platforms the mySharedLib DLL will be "
       "installed to <prefix>/bin and /some/full/path and its import library "
-      "will be installed to <prefix>/lib/static and /some/full/path. "
-      "On non-DLL platforms mySharedLib will be installed to <prefix>/lib "
-      "and /some/full/path."
+      "will be installed to <prefix>/lib/static and /some/full/path."
       "\n"
       "The EXPORT option associates the installed target files with an "
       "export called <export-name>.  "
       "It must appear before any RUNTIME, LIBRARY, or ARCHIVE options.  "
+      "To actually install the export file itself, call install(EXPORT).  "
       "See documentation of the install(EXPORT ...) signature below for "
       "details."
       "\n"
@@ -338,10 +340,12 @@ private:
   bool HandleFilesMode(std::vector<std::string> const& args);
   bool HandleDirectoryMode(std::vector<std::string> const& args);
   bool HandleExportMode(std::vector<std::string> const& args);
-  bool MakeFilesFullPath(const char* modeName, 
+  bool MakeFilesFullPath(const char* modeName,
                          const std::vector<std::string>& relFiles,
                          std::vector<std::string>& absFiles);
   bool CheckCMP0006(bool& failure);
+
+  std::string DefaultComponentName;
 };
 
 
