@@ -1,19 +1,14 @@
-/*=========================================================================
+/*============================================================================
+  CMake - Cross Platform Makefile Generator
+  Copyright 2000-2009 Kitware, Inc., Insight Software Consortium
 
-  Program:   CMake - Cross-Platform Makefile Generator
-  Module:    $RCSfile: cmLoadCacheCommand.h,v $
-  Language:  C++
-  Date:      $Date: 2005/11/23 17:33:38 $
-  Version:   $Revision: 1.11.2.1 $
+  Distributed under the OSI-approved BSD License (the "License");
+  see accompanying file Copyright.txt for details.
 
-  Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
-  See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even 
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
-     PURPOSE.  See the above copyright notices for more information.
-
-=========================================================================*/
+  This software is distributed WITHOUT ANY WARRANTY; without even the
+  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+  See the License for more information.
+============================================================================*/
 #ifndef cmLoadCacheCommand_h
 #define cmLoadCacheCommand_h
 
@@ -39,17 +34,18 @@ public:
    * This is called when the command is first encountered in
    * the CMakeLists.txt file.
    */
-  virtual bool InitialPass(std::vector<std::string> const& args);
+  virtual bool InitialPass(std::vector<std::string> const& args,
+                           cmExecutionStatus &status);
 
   /**
    * The name of the command as specified in CMakeList.txt.
    */
-  virtual const char* GetName() { return "LOAD_CACHE";}
+  virtual const char* GetName() const { return "load_cache";}
 
   /**
    * Succinct documentation.
    */
-  virtual const char* GetTerseDocumentation() 
+  virtual const char* GetTerseDocumentation() const
     {
     return "Load in the values from another project's CMake cache.";
     }
@@ -57,16 +53,16 @@ public:
   /**
    * More documentation.
    */
-  virtual const char* GetFullDocumentation()
+  virtual const char* GetFullDocumentation() const
     {
     return
-      "  LOAD_CACHE(pathToCacheFile READ_WITH_PREFIX\n"
+      "  load_cache(pathToCacheFile READ_WITH_PREFIX\n"
       "             prefix entry1...)\n"
       "Read the cache and store the requested entries in variables with "
       "their name prefixed with the given prefix.  "
       "This only reads the values, and does not create entries in the local "
       "project's cache.\n"
-      "  LOAD_CACHE(pathToCacheFile [EXCLUDE entry1...]\n"
+      "  load_cache(pathToCacheFile [EXCLUDE entry1...]\n"
       "             [INCLUDE_INTERNALS entry1...])\n"
       "Load in the values from another cache and store them in the local "
       "project's cache as internal entries.  This is useful for a project "
@@ -87,7 +83,6 @@ protected:
   
   bool ReadWithPrefix(std::vector<std::string> const& args);
   void CheckLine(const char* line);
-  bool ParseEntry(const char* entry, std::string& var, std::string& value);
 };
 
 
