@@ -43,12 +43,12 @@ public:
   /**
    * The name of the command as specified in CMakeList.txt.
    */
-  virtual const char* GetName() { return "qt_wrap_ui";}
+  virtual const char* GetName() const { return "qt_wrap_ui";}
 
   /**
    * Succinct documentation.
    */
-  virtual const char* GetTerseDocumentation() 
+  virtual const char* GetTerseDocumentation() const
     {
     return "Create Qt user interfaces Wrappers.";
     }
@@ -56,7 +56,7 @@ public:
   /**
    * More documentation.
    */
-  virtual const char* GetFullDocumentation()
+  virtual const char* GetFullDocumentation() const
     {
     return
       "  qt_wrap_ui(resultingLibraryName HeadersDestName\n"
