@@ -1,25 +1,21 @@
-/*=========================================================================
+/*============================================================================
+  CMake - Cross Platform Makefile Generator
+  Copyright 2000-2009 Kitware, Inc., Insight Software Consortium
 
-  Program:   CMake - Cross-Platform Makefile Generator
-  Module:    $RCSfile: cmSiteNameCommand.cxx,v $
-  Language:  C++
-  Date:      $Date: 2004/04/15 15:38:35 $
-  Version:   $Revision: 1.21 $
+  Distributed under the OSI-approved BSD License (the "License");
+  see accompanying file Copyright.txt for details.
 
-  Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
-  See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even 
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
-     PURPOSE.  See the above copyright notices for more information.
-
-=========================================================================*/
+  This software is distributed WITHOUT ANY WARRANTY; without even the
+  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+  See the License for more information.
+============================================================================*/
 #include "cmSiteNameCommand.h"
 
 #include <cmsys/RegularExpression.hxx>
 
 // cmSiteNameCommand
-bool cmSiteNameCommand::InitialPass(std::vector<std::string> const& args)
+bool cmSiteNameCommand
+::InitialPass(std::vector<std::string> const& args, cmExecutionStatus &)
 {
   if(args.size() != 1 )
     {
@@ -35,13 +31,13 @@ bool cmSiteNameCommand::InitialPass(std::vector<std::string> const& args)
   paths.push_back("/usr/local/bin");
   
   const char* cacheValue
-    = m_Makefile->GetDefinition(args[0].c_str());
+    = this->Makefile->GetDefinition(args[0].c_str());
   if(cacheValue)
     {
     return true;
     }
   
-  const char *temp = m_Makefile->GetDefinition("HOSTNAME");
+  const char *temp = this->Makefile->GetDefinition("HOSTNAME");
   std::string hostname_cmd;
   if(temp)
     {
@@ -55,9 +51,9 @@ bool cmSiteNameCommand::InitialPass(std::vector<std::string> const& args)
   std::string siteName = "unknown";
 #if defined(_WIN32) && !defined(__CYGWIN__)
   std::string host;
-  if(cmSystemTools::ReadRegistryValue(
-    "HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\ComputerName\\ComputerName;ComputerName",
-    host))
+  if(cmSystemTools::ReadRegistryValue
+     ("HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\"
+      "Control\\ComputerName\\ComputerName;ComputerName", host))
     {
     siteName = host;
     }
@@ -67,7 +63,7 @@ bool cmSiteNameCommand::InitialPass(std::vector<std::string> const& args)
     {
     std::string host;
     cmSystemTools::RunSingleCommand(hostname_cmd.c_str(),
-      &host, 0, 0, false);
+      &host, 0, 0, cmSystemTools::OUTPUT_NONE);
     
     // got the hostname
     if (host.length())
@@ -88,7 +84,7 @@ bool cmSiteNameCommand::InitialPass(std::vector<std::string> const& args)
       }
     }
 #endif
-  m_Makefile->
+  this->Makefile->
     AddCacheDefinition(args[0].c_str(),
                        siteName.c_str(),
                        "Name of the computer/site where compile is being run",
