@@ -1,16 +1,14 @@
-/*=========================================================================
+/*============================================================================
+  KWSys - Kitware System Library
+  Copyright 2000-2009 Kitware, Inc., Insight Software Consortium
 
-  Program:   KWSys - Kitware System Library
-  Module:    $RCSfile: Terminal.c,v $
+  Distributed under the OSI-approved BSD License (the "License");
+  see accompanying file Copyright.txt for details.
 
-  Copyright (c) Kitware, Inc., Insight Consortium.  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notices for more information.
-
-=========================================================================*/
+  This software is distributed WITHOUT ANY WARRANTY; without even the
+  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+  See the License for more information.
+============================================================================*/
 #include "kwsysPrivate.h"
 #include KWSYS_HEADER(Terminal.h)
 
@@ -141,17 +139,41 @@ static const char* kwsysTerminalVT100Names[] =
   "con80x43",
   "con80x50",
   "con80x60",
+  "cons25",
   "console",
   "cygwin",
+  "dtterm",
+  "eterm-color",
+  "gnome",
+  "gnome-256color",
   "konsole",
+  "konsole-256color",
+  "kterm",
   "linux",
   "msys",
+  "linux-c",
+  "mach-color",
+  "mlterm",
+  "putty",
   "rxvt",
+  "rxvt-256color",
+  "rxvt-cygwin",
+  "rxvt-cygwin-native",
   "rxvt-unicode",
+  "rxvt-unicode-256color",
   "screen",
+  "screen-256color",
+  "screen-256color-bce",
+  "screen-bce",
+  "screen-w",
+  "screen.linux",
   "vt100",
   "xterm",
+  "xterm-16color",
+  "xterm-256color",
+  "xterm-88color",
   "xterm-color",
+  "xterm-debian",
   0
 };
 
